@@ -92,6 +92,7 @@ fn ctx.f_blkstr(str, pos, resc, rend, ctg) {
 
 // fc("(-....\(...-)", 1, '\\', "-)" ) で-)の")"位置を返す
 // escとendはluaパターン eposは]の一番最後を示す
+// rescが""ならesc無しで純粋にrendを探す
 fn ctx.f_blkend(str, pos, resc, rend) {
 	resc=resc||""
 	local spos=pos
@@ -420,7 +421,6 @@ fn ctx.f_regstr(str, pos, resc, rend, ctg) {
 			//hit [  加工して\123系は外にだす
 			rs = rs..string.sub(str, pos, sss-1)
 			//normalは普通に追加しとく
-			
 			local res, emsg = ctx.getclass(str, sss)
 //print(res, emsg, "e_")
 			if(emsg){
@@ -642,6 +642,9 @@ fn ctx.f_scanpeg(data, exp) {
 			if(rs=='r"') {	rs,cur,ctg = ctx.f_regstr(str, cur+2, "[\\]", '["]', "reg"); break }
 			if(rs=="r'") {	rs,cur,ctg = ctx.f_regstr(str, cur+2, "[\\]", "[']", "reg"); break }
 			
+			//heredoc
+			if(rs=='h"') {	rs,cur,ctg = ctx.f_blkstr(str, cur+2, "", '["]', "hlit"); break }
+			if(rs=="h'") {	rs,cur,ctg = ctx.f_blkstr(str, cur+2, "", "[']", "hlit"); break }		
 			//lit,class, multi word. cur/escreg/endreg
 			rs=string.sub(str,cur,cur)
 			if(rs=="'") {	rs,cur,ctg = ctx.f_blkstr(str, cur+1, "[\\]", "[']", "lit"); break }
@@ -687,9 +690,9 @@ fn ctx.f_scanpeg(data, exp) {
 			break
 		}
 		//err
-	
 		if(aflg<0){ctg="err"; rs="uneven curly braces"}
 		if(ctg == "err" ) {	return nil, ctx.srcemsg(str, cur, rs) }
+		//charesc
 		if( !string.match(exp, "[rR]",1) ){
 			//拡張モード以外はオリジナルだけ
 			if( string.find("{} aE aC = reg sep", ctg, 1, true)&&grule ){
@@ -771,7 +774,7 @@ fn ctx.f_scanpeg(data, exp) {
 					num = tonumber(string.sub(sbuf, 2), 8)
 					cstr = string.char(num)
 					//\ooo >> 1char >> add \\ if needed
-					if(rcls<0&&string.find(".[$()|*+?{\\",cstr,1,true) ){cstr="\\"..cstr}
+					if(rcls<0&&string.find("^.[$()|*+?{\\",cstr,1,true) ){cstr="\\"..cstr}
 					if(ctg=="class"&&cstr=="-"){
 						//peg-class hyphen問題は-のみ\[は1byteでいい。oct-は小細工が必要
 						//octの\055 - を始末 三連を頭にセット trick
@@ -810,7 +813,7 @@ fn ctx.f_scanpeg(data, exp) {
 						if(ctg=="class"){ return nil, ctx.srcemsg(str, cur-(string.len(rs)-cpos)-2, "peg-class unsupports unicode: "..sbuf) }
 						c = u.strconv(sbuf, "uc:b")
 						// reg系機能付きはesc pclsは弾いてる 全て変換してよし
-						if(rcls<0&&string.find(".[$()|*+?{\\", c, 1, true) ){c="\\"..c}
+						if(rcls<0&&string.find("^.[$()|*+?{\\", c, 1, true) ){c="\\"..c}
 						s = s..c
 						cpos = cpos+string.len(sbuf)
 						goto lb_NEXT
@@ -833,6 +836,7 @@ fn ctx.f_scanpeg(data, exp) {
 		//classは追加変換したいけどtermsが出来て!. EOFが必要なので後回し
 		local term = { ["ctg"]=ctg, ["data"]=s, ["info"]=ctg..":"..s, ["pos"]=cur }
 		if(ctg=="class"&&s=="."){ term.ctg = "any" }
+		elif(ctg=="hlit"){ term.ctg = "lit" }
 		elif(ctg=="class"){
 			//class dataは255のbinになる
 			local res, str = ctx.convclass(term.data)
@@ -1059,6 +1063,13 @@ u.laptime()
 //	obj, str = m.f_scanpeg(str, "rN")
 //	print(str)
 //	u.test_eq( obj[3].data, "1", str)	//-
+
+//heredoc
+	obj, str = m.f_scanpeg([[ id <- h'a\\\134[u3042\]"' ]], "r")
+	u.test_eq(obj[3].data, /*=a\\\134[u3042\]"=*/)	//litは\]が]になる
+
+	obj, str = m.f_scanpeg([[ id <- h"a\\\134'\'"]], "r")
+	u.test_eq(obj[3].data, /*=a\\\134'\'=*/)	//litは\]が]になる
 }
 //SH_TSE
 
@@ -1296,6 +1307,7 @@ fn ctx.frulesinfo(fh, rules){
 			// esepで終わるから体裁はそれなりに整う
 		}
 	}
+	res = string.gsub(res, "%z", "\\0")
 	u.fprintf(fh, "%s", res)
 	return res
 }
@@ -1528,37 +1540,63 @@ fn u.addtest.t_taperules() {
 }
 //SH_TSE
 
-// pegruleのjmploopのテスト. identと*と""の無限を調べる。それ以外はposが進むのでok
-// jmploopは致命傷だけど*の未消費は限りなく黒だけどスルー。
-// スタンプラリーで訪れた場所をメモってく。jmpstkでネストを管理。
+// jmpチェック. identと''*の無限を調べる。それ以外は進む可能性が残る
+// *系は限りなく黒だけどスルー 実行時検査に任せる
 // rootrulesを使ってたおかげでそのまま使える。助かった
-// loopckも2nd rtnにおまけデータでpass系rnumが怪しい奴がtbで入ってる
+// 追加検査: !& *?の競合''と!''に縮退するので文法エラーとするのが正しい
+// !+と&+は無意味だからこいつもついでにkickする
 fn ctx.loopck(rules) {
 	//枝分かれとerr記録用。err以外は巻き戻したりする
-	local st={rpos=1, tpos=1, eres="", looptb={}, jmpstk={}, rc=0, passrule={} }
+	local st={rpos=1, tpos=1, eres="", looptb={}, jmpstk={}, rc=0 }
 	local buf=nil
 	st.eres=st.eres.."> -- rule 1 "..rules[st.rpos][1].data
 	while(1){
 		local term=rules[st.rpos][st.tpos]
 		if(st.tpos==1){ st.looptb[term.data]=1; goto lb_WALK }
-		if(term.ctg=="act"){ goto lb_WALK }
-		if(st.rc==0 && term.ctg=="ident"){
+		if(st.tpos>=2&&term.ctg!="ident"&&term.pre==nil){
+			//進行要素があればリセット
+			st.looptb[rules[st.rpos][1].data]=nil
+		}
+		//term検査を追加 badfixをkickする !&*+?は択一にする
+		if( term.pre && term.suf){
+			st.eres=st.eres..
+			u.fprintf(nil, "\nERR: rule %d %s: %d,%d %s\n"..
+			"terms using both '!&' and '?*' is valid PEG but must be illegal rule\n" ..
+			"   !__*, !__? is always fail same as !''	eg) R <- !'a'*	#>>'a', 'x'>>fail\n"..
+			"   &__*, &__? is always suc  same as  ''	eg) R <- &'a'?	#>>'a', 'x'>>suc\n"..
+			"\n'!&' and '+' is used only in rare cases, so split the rules if necessary\n"..
+			"	eg) R<- !'a'+ 	>>>		R1 <- !R2	R2<- 'a'+"
+			, st.rpos, rules[st.rpos][1].data, st.rpos, st.tpos-1, term.info)
+			return nil, st.eres
+		}
+		//空文字loopは即死
+		if( (term.suf=="*"||term.suf=="+")&&(term.ctg=="lit"&&term.data=="") ){
+			st.eres=st.eres..
+			u.fprintf(nil, "\nERR rule %d %s: %d,%d: bad peg. empty lit loop"
+			, st.rpos, rules[st.rpos][1].data, st.rpos, st.tpos-1)
+			return nil, st.eres
+		}
+		//本来の検査
+		if(term.ctg=="ident"){
+			//事前に消費要素があってidentならスキップで*?扱い
+			//結果の再利用はしないから少し無駄があるけど面倒なので放置
+			if( st.looptb[rules[st.rpos][1].data]==nil ){goto lb_WALK}
+			//アウト
 			if(st.looptb[term.data]){
-				st.lpflg=1
 				local dst
 				for(i,v in ipairs(rules) ){	if(v[1].data==term.data){dst=i;break} }
 				st.eres=st.eres..
 				u.fprintf(nil, ": bad peg. infinite ruleloop: rule %d %s: %d,%d >> rule %d %s"
-				, st.rpos, rules[st.rpos][1].data, st.rpos, st.tpos, dst, rules[dst][1].data)
+				, st.rpos, rules[st.rpos][1].data, st.rpos, st.tpos-1, dst, rules[dst][1].data)
 				return nil, st.eres
 			}
-			//jmp
+			//loop候補 初jmp
 			st.jmpstk[#st.jmpstk+1] = st.rpos
 			st.jmpstk[#st.jmpstk+1] = st.tpos
 			local rname = rules[st.rpos][st.tpos].data
+			//未定義検査 逆引き
 			for(i,v in ipairs(rules) ){ if(v[1].data==rname){rname=i; break} }
 			if(type(rname)!="number"){
-				//逆引き出来ない==ruleが無い
 				st.eres=st.eres..
 				u.fprintf(nil, ": bad peg. using nodef rule: rule %d %s: %d,%d >> %s"
 				, st.rpos, rules[st.rpos][1].data, st.rpos, st.tpos-1, rname)
@@ -1569,38 +1607,32 @@ fn ctx.loopck(rules) {
 			st.eres=st.eres.."\n> jmp rule "..tostring(st.rpos)..": "..rules[st.rpos][1].data
 			goto lb_NEXT
 		}
-		if(term.suf=="*"||term.suf=="?"||term.pre!=nil||(term.ctg=="lit"&&term.data=="")||term.ctg=="reg" ){
-			st.rc= -1	//未消費マーク reg追加
-			goto lb_WALK
-		}
-		else{	st.rc=1 } //消費マーク
-	//	if(term.ctg=="lit"||term.ctg=="eof"){	st.rc=1 } //消費マーク
-
-		if(st.rc==1){ goto lb_WALL }
-		if(st.rc!=1){goto lb_WALK}		//nohitでfinish
+		// identはpreがあっても必須それ以外は preと''は未消費なのでスキップ
+		if(term.pre!=nil||(term.ctg=="lit"&&term.data=="")){ goto lb_WALK }
+	::lb_NOLP::
+		//おそらく消費があった jmp/skip系はWALKかNEXTしてる
+		st.eres=st.eres.."\n> jmp stop "..tostring(st.rpos)
+		if(#st.jmpstk!=0){ st.rpos=st.jmpstk[1]}
+		goto lb_RNEXT
 	::lb_WALK::
 		st.tpos=st.tpos+1
-		if( st.tpos <= #rules[st.rpos] ){
-			st.rc=0;	//歩くなら初期化
-			goto lb_NEXT
-		}
-		//歩けないなら始末
+		if( st.tpos <= #rules[st.rpos] ){ goto lb_NEXT }
+		//終端なら判定が必要 jmpで次に行くかもしれない
 		goto lb_WALL
 	::lb_WALL::
-		//ここにくるのは0jmpか未消費終端か。
-		//regexはtapeしてからcでfastmapするので怪しい判定しか出来ない
-		if(st.rc!=1){
-			st.eres=st.eres.."\ninfo: rule "..st.rpos.." "..rules[st.rpos][1].data..": regex / all token includes skip factor (!&*? '')"
-			st.passrule[st.rpos]=1
-			st.rc= -1
-		}
+		// >>jmp中ならnextを調べないといけない
 		if(#st.jmpstk!=0){
-			st.eres=st.eres.."\n> rtn rule "..tostring(st.rpos)
-			st.looptb[rules[st.rpos][1].data]=nil;
-			st.tpos=st.jmpstk[#st.jmpstk]; st.jmpstk[#st.jmpstk]=nil
-			st.rpos=st.jmpstk[#st.jmpstk]; st.jmpstk[#st.jmpstk]=nil
-			goto lb_NEXT
+			//nextが居た
+			if(rules[st.rpos+1][1].data == rules[st.rpos+1][1].data){
+				st.rpos=st.rpos+1
+				st.tpos=1
+				st.eres=st.eres.."\n> subrule "..tostring(st.rpos)
+				goto lb_NEXT
+			}
+			//いなかったらloop無し確定
+			goto lb_NOLP
 		}
+		//jmp中じゃなかった フツーにok
 		goto lb_RNEXT
 	::lb_RNEXT::
 		st.rpos=st.rpos+1
@@ -1612,12 +1644,11 @@ fn ctx.loopck(rules) {
 		st.tpos=1
 		st.looptb={}
 		st.jmpstk={}
-		st.rc=0
 		st.eres=st.eres.."\n> -- rule "..tostring(st.rpos).." "..rules[st.rpos][1].data
 	::lb_NEXT::
 	}
 	st.eres=st.eres.."\nloop check ok\n"
-	return st.eres, st.passrule
+	return st.eres
 }
 // 1stはinfostr, 2ndは未消費の危険がある奴等のflg, r[10]=1 , rule10が*のみとか
 
@@ -1629,6 +1660,8 @@ fn u.addtest.t_loopck() {
 	/ unko [abc]
 	unko <- "123"
 =*/
+	goto lb_TGT;
+
 	tb, str = m.f_scanpeg(bstr, "r")
 u.dbg(tb, str)
 	rules, emsg = m.bldrules(str, tb)
@@ -1654,6 +1687,63 @@ u.dbg(tb, str)
 	rc, emsg = m.loopck(rules)
 	rc=rc||u.errstop(emsg)
 print(rc)
+	
+//	bstr= "id <- !'a'?"
+	bstr= "id <- ''+"
+	tb, str = m.f_scanpeg(bstr, "r")
+	rules, emsg = m.bldrules(str, tb)
+	rc, emsg = m.loopck(rules)
+	u.test_eq(rc, nil, emsg)
+
+	bstr= "id <- ''*"
+	tb, str = m.f_scanpeg(bstr, "r")
+	rules, emsg = m.bldrules(str, tb)
+	rc, emsg = m.loopck(rules)
+	u.test_eq(rc, nil, emsg)
+
+	bstr= "id <- !'a'?"
+	tb, str = m.f_scanpeg(bstr, "r")
+	rules, emsg = m.bldrules(str, tb)
+	rc, emsg = m.loopck(rules)
+	u.test_eq(rc, nil, emsg)
+
+	bstr= "id <- !'a'*"
+	tb, str = m.f_scanpeg(bstr, "r")
+	rules, emsg = m.bldrules(str, tb)
+	rc, emsg = m.loopck(rules)
+	u.test_eq(rc, nil, emsg)
+
+	bstr= "id <- !'a'+"
+	tb, str = m.f_scanpeg(bstr, "r")
+	rules, emsg = m.bldrules(str, tb)
+	rc, emsg = m.loopck(rules)
+print(emsg)
+	u.test_eq(rc, nil, emsg)
+
+::lb_TGT::;
+	bstr= " r1 <- !'a' r2"..
+		" r2 <- &'' r1 "
+	tb, str = m.f_scanpeg(bstr, "r")
+	rules, emsg = m.bldrules(str, tb)
+	rc, emsg = m.loopck(rules)
+print(emsg)
+	u.test_eq(rc, nil)
+
+	bstr= " r1 <- !'a' r2"..
+		" r2 <- 'b'+ r1 "
+	tb, str = m.f_scanpeg(bstr, "r")
+	rules, emsg = m.bldrules(str, tb)
+	rc, emsg = m.loopck(rules)
+	u.test_eq(rc!=nil)
+
+	bstr= " r1 <- !'a' r2"..
+		" r2 <- !'b' / 'b'* r1 "
+	tb, str = m.f_scanpeg(bstr, "r")
+	rules, emsg = m.bldrules(str, tb)
+	rc, emsg = m.loopck(rules)
+//print(rc, emsg)
+	u.test_eq(rc!=nil)
+
 //	if(emsg){ u.perr(emsg) }	//警告出力
 // os.exit(1)
 }
@@ -1804,7 +1894,7 @@ There is NO WARRANTY, to the extent permitted by law.=*/
 fn ctx.ped_help(){
 	local buf=/*=
 HowTo (ped, sed-style editer using peg)	
-opt: -[e|f][n|N][r|R] (-hHgV, -tTodEL: other opt. see ~$ ped -H|less ) 
+opt: -[e|f][n|N][r|R] ( -hHgVtTodEL: others. see ~$ ped -H|less ) 
  e/f(pegexpr/file), n/N(noout/NotALLOW if hit norule), r/R(expand syntax)
 --
  eg) ~$ echo "12a"|ped -re 'ID<-[a-z]+ {_1="Z"}'	#-enf works like sed opt
@@ -2223,6 +2313,20 @@ u.run_lutest(_G.arg)
 /*
  change log
  --
+2021-07-26  Momi-g	<dmy@dmy.dmy>
+
+	* *SH_bn*.sh.lua (frulesinfo): replace infostr \0 >> '\\0' for disp msg
+	
+	* *SH_bn*.sh.lua (scanpeg): fix reg-octesc 'hat' ".[$(..." >> "^.[$(..."
+
+2021-07-21  Momi-g	<dmy@dmy.dmy>
+
+	* *SH_bn*.sh.lua (loopck): change loopck routine, add -2, fillrpos etc
+
+2021-07-15  Momi-g	<dmy@dmy.dmy>
+
+	* *SH_bn*.sh.lua (scanpeg): add heredoc h""/h''
+
 2021-07-10  Momi-g	<dmy@dmy.dmy>
 
 	* *SH_bn*.sh.lua (loopck): add undef rule using emsg
