@@ -148,6 +148,13 @@ _fail
 ped -re 'R1 <- "v" id' tests.sh
 _fail
 
+ped -e 'R1 <- h"\"' tests.sh
+_suc
+
+buf='\\'
+buf=`ped -re 'R1 <- h"\" {_0=h"@"}' -E "$buf"`
+_eq '@@' "$buf" 
+
 ped -re '' tests.sh > buf.txt
 _suc
 diff -u tests.sh buf.txt
