@@ -316,7 +316,7 @@ typedef struct atoken_tag {
 	int rnum;
 	const char* rname;
 } atoken_t;
-typedef char* clist_t;
+typedef char clist_t[256];
 
 typedef struct peg_rulesdata_tag{
 	rtoken_t* rtokens;	//-1 -9, -9-9のセンチネル付きだったはず
@@ -449,7 +449,8 @@ static rdata_t* rdata_new(const char* mode, const char* rstr, int rstrsz, const
 static void rdata_free_impl(rdata_t* obj);
 static char* set_rglists(rdata_t* obj) ;
 static void reset_rglists(rdata_t* rdata);
-static const char* set_clists(rdata_t* rdata);
+static char* set_clists(rdata_t* rdata);
+static char* clists_info(ped_t* ped);
 static void reset_clists(rdata_t* rdata);
 static jst_t* jst_new(const char* nlstr, int nlstrsz);
 #define jst_free(obj)	(jst_free_impl(obj), obj=NULL)
@@ -519,9 +520,8 @@ ped_t* ped_new_impl(const char* mode, const char* rstr, int rstrsz, const char*
 	if( strchr(mode,'h') ){ ped->ruleinfo = ped_info("h", rstr, rstrsz); }
 	else if( strchr(mode,'H') ){ ped->ruleinfo = ped_info("H", rstr, rstrsz); }
 	else if( strchr(mode,'V') ){ ped->ruleinfo = ped_info("V", rstr, rstrsz); }
-	else if( strchr(mode,'d') ){ ped->ruleinfo = ped_info(mode, rstr, rstrsz); }
+	else if( strchr(mode,'d') ){ ped->ruleinfo = ped_info(mode, rstr, rstrsz);}
 	if( strspn(mode, "hHV") ){ return ped; }
-
 //maindata
 	ped->msv = msv_new(mode, nlstr, nlstrsz);
 //	
@@ -529,10 +529,21 @@ ped_t* ped_new_impl(const char* mode, const char* rstr, int rstrsz, const char*
 	ped->rdata = rdata_new(mode, rstr, rstrsz, nlstr, nlstrsz);
 	if(errno){
 		//emsgはmemされてる
-		ped->rdata=NULL;
 		emsg=(char*)ped->rdata;
+		ped->rdata=NULL;
 		goto lb_ERR;
 	}
+	//情報追加
+	if( strchr(mode,'d') ){
+		char* s = (char*)ped->ruleinfo;
+		char* ss = clists_info(ped);
+		int len = strlen(s)+ strlen(ss);
+		char* p = malloc(len+20);
+		sprintf(p, "%s\n--fastmap_info\n%s", s, ss);
+		free(s);
+		free(ss);
+		ped->ruleinfo = p;
+	}
 //
 	ped->jst = jst_new(ped->rdata->nlstr, ped->rdata->nlstrsz);
 	if(errno){ emsg=(char*)ped->jst; goto lb_ERR; }
@@ -869,7 +880,8 @@ static rdata_t* rdata_new(const char* mode, const char* rstr, int rstrsz, const
 //dbg(obj);
 	emsg = set_rglists(obj);
 	if(emsg){ goto lb_ERR; }	//emsgはmemされてる
-	set_clists(obj);
+	emsg = set_clists(obj);
+	if(emsg){ goto lb_ERR; }	//emsgはmemされてる
 //
 	int rpos=0;
 	for(;;rpos++){
@@ -1001,7 +1013,8 @@ static void reset_rglists(rdata_t* rdata) {
 	free(rdata->rglists);
 }
 
-static const char* set_clists(rdata_t* rdata) {
+static char* set_clists(rdata_t* rdata) {
+	char* emsg=NULL;
 	/*
 	ID <- "123"	...1
 	ID <- "123"* "xyz"	...1,x
@@ -1012,114 +1025,285 @@ static const char* set_clists(rdata_t* rdata) {
 	ID <- jmp*	"xyz"	...jmp, x
 	ID <- ""			...all
 	ID <- jmp*	""		...all
+	ID <- !jmp	(end)	...reverse all	//add,fix
 		//eofは成功者なしということになる。left>0で絞ることにする
-		//逆に文字があったら自動失敗だ。
-		//未消費可能なruleは全許可 無限ループはsuc判定で弾く
-
-		//listの戦略は99%のruleは失敗で1%は成功だから成功の頭一文字で
-		//フィルターする。完全で無いけど80%ぐらいは削減というもの。
-		//hit先頭文字を集める.
+		//未消費系は全許可 無限ループはclistと実行時両方で弾く
+		//戦略は99%のruleは失敗で1%は成功だから一文字で多くの失敗を弾ける
+		//疑陽性も出るが80%ぐらいは削減 0は確実に失敗 -1は確実に未消費成功
 	*/
-	//idxはrheadでluaになってるので-1でcに合わせる
-	//まずはlit,reg系だけ集める >> any eof classが追加されたident以外を集める
-	//関門文字を決定したあとでskip系を追加する
-
-	//未消費系のマーキング 最後で使う
+	// !系は 判定が複雑な割には効率が悪いのでスキップ classだけは使える
+	// *?系は勘定に入るが後続も続く 無しならそこでstop 最後まで行ったら-1
+	// jmpは再起jmpで葉まで探索してから遡上する 合成時はnextもチェックする
+	// !''*+系は弾いてるので存在しない >> 疑陽性が出るので実行時もloop検査
+	// 単純結合だとidentが-1か1か不明なので合成時にデッドロック的な検査がいる
+	
 	int erpos = 0;	//内部ruleも含めた全ルールのsentinel
 	for(; rdata->rhead[erpos]!= -9; erpos++);;
-	int* allmk = calloc(sizeof(int), erpos);	//esepはclist無し
 
-	//pass1 下調べ
-	dq_t* obj = pdq_new();
+	//メモバッファ準備
+	//typedef char (*fm_t)[256];	//fastmap
+	typedef char fm_t[256];
+	fm_t* clists = (fm_t*)calloc(256*erpos, sizeof(char) );
+	char* lprpos = (char*)calloc(erpos, sizeof(char) );	//loop
+	char* fillrpos = (char*)calloc(erpos, sizeof(char) );	//carrが完了したruleflg
+	dq_t* rstk = idq_new();	//identで飛ぶ度にpush. orを見るときはスルー
+	dq_t* tstk = idq_new();
+//
 	for(int rpos=0; rpos<erpos; rpos++) {
-		//bit演算を使えばflg系が少し楽
-		char* arr = (char*)calloc(256, sizeof(char));
-		int idx = rdata->rhead[rpos] -1;	//lua分 
-		int tpos = 1; 	//1スタート 0はheadが入る
+		if(fillrpos[rpos]){continue;}	//立ち寄りで始末済
+		int idx = rdata->rhead[rpos] -1;	//lua分 rpos0はheader
+		int tpos = 1; 	// 1スタート 0はheadが入る
 		for(;;tpos++){
+lb_JMP:;
+			char abuf[256]={0};
 			rtoken_t tk = rdata->rtokens[idx+tpos];
-//dbg(tk.ctg, tk.data, (unsigned char)tk.data[0], tk.num);
-//sleep(3);
-			// 先頭になれる文字たちを集める
-			if(tk.pre[0]=='!') { continue;}
-			if(tk.ctg[0]=='l'&&tk.num>0) { arr[(unsigned char)tk.data[0]]=1; }
-			//tk.dataはchar*でマイナスになるのでcastが必要 ハマった
-			//ureg_t* 
-			//rglistはrtokensの羅列idxをNULL付きで一列にrgdataを入れてる
-			if(tk.ctg[0]=='r') {
-				char* fmap = rdata->rglists[idx+tpos]->fastmap;
-				 //gnu-regex fastmap casts char -> unsigned int in innercode
-				CBIT_SET(arr, fmap, 256);
-			}
-			// classとanyはtkdataに0-255までのbit羅列を事前に用意してる
-			// litと違って順序情報がいらずdataスペースが空いてるため
-			if(tk.ctg[0]=='c'||tk.ctg[0]=='a') { CBIT_SET(arr, tk.data, 256); }
-			if(tk.suf[0]=='*'||tk.suf[0]=='?'||(tk.ctg[0]=='l'&&tk.num==0)) {continue;}
-			//NEXT可能なのは*?復活系と""!だけ。&はそこで終了
-			
-			//全部*?系で未消費でオワタ >>sepに突入するので自動break
-			if(tk.ctg[0]=='s') {allmk[rpos]=1;}	//未消費可能性を立てておく
-			//残骸はsepとかesep,eofもあるけどcharカウントには無関係
-			break;
-		}
-		pdq_push(obj, arr);
-	}
-	char** clists = (char**)pdq_2arr(obj);
-	pdq_free(obj);
-	//clistはidentの部分が入ってないので合わせて合成する
-	int cnt=0;
-	while(1) {
-		int lpflg=0;
-		for(int rpos=0; rdata->rhead[rpos]!= -9; rpos++) {
-			int idx = rdata->rhead[rpos] -1;	//lua分
-			int tpos=1;	//0はheadが入ってるのでスキップ
-			for(;;tpos++) {
-				rtoken_t tk = rdata->rtokens[idx+tpos];
-				if(tk.pre[0]=='!') { continue;}
-				//追加対象はidentのみ
-				if(tk.ctg[0] == 'i') {
-					int dst = tk.num -1;	//lua基準, 名前が同じなら拾っていく
-					while(1){
-						for(int ii=0; ii<256; ii++) {
-							//追加変更有り
-							if( clists[rpos][ii]==0 && clists[dst][ii]) {
-								clists[rpos][ii]=1; lpflg=1;
-							}
+			// !系はlit/reg/identで意味を持たない。![]だけは手がかりにつかえる
+			if(tk.ctg[0]=='i') {
+				if(tk.pre[0]=='!'){	continue; }
+				//pass系は0が可能性が残ってる. 最後までいったら-1フラグ				
+				int dst=tk.num-1;
+				const char* dstname = rdata->rtokens[rdata->rhead[dst]-1].data;
+				//下準備が終わってるなら追加してく
+				while( fillrpos[dst] ){
+					for(int i=0;i<256;i++){ if(clists[dst][i]==1){clists[rpos][i]=1; } }
+					dst++;	//共通で使うので先にdef
+					const char* p="";
+					if(rdata->rhead[dst]>0) { p=rdata->rtokens[rdata->rhead[dst] -1].data; }
+					if( fillrpos[dst-1]== -1 ){
+						//本来のjmp先 上で++してるので--辻褄合わせ
+						//未消費系は伝染する
+						fillrpos[rpos]= -1;
+						if(strcmp(dstname,p)==0 ){
+						//未消費系なのに後ろが居る 絶対に到達しないerr
+				char sbuf[256]={0};
+				sprintf(sbuf
+				, "ERR: rule %s:%d,%d: unreachable rule. use cmtout '#'"
+				, p, dst, 0);
+							emsg=strdup(sbuf);
+							free(clists);
+							clists=NULL;
+							goto lb_RTN;			
 						}
-						dst++;
-						//同名からも回収
-						int nidx = rdata->rhead[dst]-1;	//lua分
-						if(nidx<0){break;} //ruleからidxを取るけどendなら終
-						// fix idx>>nidx, typo
-						const char* s = rdata->rtokens[nidx].data;
-						if( strcmp(tk.data, s)!=0){break;}
-						//同名でないidentなら終わり
+						//未消費系はnext
+						tpos++;
+						goto lb_JMP;
+					}
+					//消費系でも後ろがあるなら追加する
+					if(strcmp(dstname,p)==0 && fillrpos[dst] ){ continue; }
+					if(strcmp(dstname,p)==0 ){
+						//同名だけど未挑戦 >>見に行って戻る
+						//重複になるけどloopで潰す
+						idq_push(rstk, rpos);
+						idq_push(tstk, tpos);
+						//lprpos[rpos]=1;	//countは初回にやってるので不要
+						rpos=dst;
+						tpos=0+1;	//+tpos_headはforがある
+						idx = rdata->rhead[rpos] -1;	//lua -1 
+						goto lb_JMP;
+					}
+					//全員消費系で完了 >> *があるかもしれない
+					fillrpos[rpos]= 1;
+					goto lb_WALK;
+				}
+				//初回系
+				if( lprpos[tk.num-1] ==0 ){
+					//初物 現在位置メモ
+					idq_push(rstk, rpos);
+					idq_push(tstk, tpos);
+					lprpos[rpos]=1;
+					rpos=tk.num-1;
+					tpos=0;	//+tpos_headはforがある
+					idx = rdata->rhead[rpos] -1;	//lua -1 
+					continue;
+				}
+				//以下loop検知** lprposから関連clistが必須系になる
+				//先頭char達以外だとloopしてしまうため後続は関係なくなる
+				//再帰下降パーサの仕組みからしてloop脱出のメモ参照脱出は違法か
+				//()のネストとかは再帰で処理できるのでセマンティクスが違っちゃう
+
+				//tk.num-1がhitってことは1stに戻る末端でhit rposとtposを巻き戻す
+				//1stからfillrposは完了しないと立たないから位置はここでok
+				idq_push(rstk, rpos);	//泥縄だけどnowが入ってないので追加
+				for(int i=0; i<idq_len(rstk); i++){
+					int rnum = idq_see(rstk, i);
+					for(int ii=0;ii<256;ii++){ abuf[ii]=abuf[ii]||(clists[rnum][ii]&&1); }
+					//loopってことはどれもsepにたどり着いてないので-1が存在しない
+				}
+				//せっかく調査したけど各ruleで*?系の判定してないから毎回調査
+				//>>NO loop判定されたruleは先頭が選ばれし勇者でないと無限になる
+				// 1とは別に-2を入れて自動エラー発生させるか inputがおかしいとか
+				
+				//1以外の先頭入力は即死エラー -2
+				for(int i=0; i<256; i++){ abuf[i]!=1 ? abuf[i]= -2: 0; }
+				//全員に配布 fillも埋める
+				for(int i=0; i<idq_len(rstk); i++){
+					int rnum = idq_see(rstk, i);
+					memcpy(clists[rnum], abuf, 256);
+					fillrpos[rnum] = -2;	//消費しないとloopなので-2
+				}
+				//reset
+				rpos = idq_see(rstk, 0);
+				tpos = idq_see(tstk, 0);
+				for(int i=0; i<idq_len(rstk); i++){lprpos[idq_see(rstk, i)]=0; }
+				idq_reset(rstk);
+				idq_reset(tstk);
+				//clearして*?検討 tkが出先なので戻す
+				idx = rdata->rhead[rpos] -1;	//lua -1
+				tk = rdata->rtokens[idx+tpos];
+				break;
+				//goto lb_WALK;
+				//identとしては消費系だけど*?がついているかもしれない
+				//完了してるのでbreakです
+			}
+			else if(tk.ctg[0]=='c'||tk.ctg[0]=='a') {
+				memcpy(abuf, tk.data, 256);
+				//classのみ!系があっても1byteなのでok
+				if(tk.pre[0]=='!'){ CBIT_NOT(abuf, 256);}
+				for(int i=0;i<256;i++){
+					if(clists[rpos][i]!=1){ clists[rpos][i] = !!abuf[i]; }
+				}
+			}
+			else if(tk.ctg[0]=='l') {
+				if(tk.pre[0]=='!'&&tk.num!=0){ continue; }	// !系は分からない
+				; STOP(tk.suf[0]&&tk.pre[0], "fatal, pre+suf detect");
+				if(tk.pre[0]=='!'&&tk.num==0){
+					// !''は完全失敗。!&と*+?はloopckで弾いてるので後続はない
+					memset(clists[rpos], 0, 256);
+					; STOP(tk.suf[0], "fatal, !''+ detect?");
+				}
+				//無条件成功なので自動的に次に回す
+				else if(tk.num==0){ continue; }
+				else {
+					//フツーのlit系。+*が居るかも
+					//tk.dataはchar*でマイナスになるのでcastが必要 ハマった
+					int c = (unsigned char)tk.data[0];
+					clists[rpos][c]=1;
+				}
+			}
+			else if(tk.ctg[0]=='r') {
+				if(tk.pre[0]=='!'){	continue; }
+				//rglistはrtokensの羅列idxをNULL付きで一列にrgdataを入れてる 殆ど空だけど
+				char* fmap = rdata->rglists[idx+tpos]->fastmap;
+				// ''系も壁も居るが検査
+				int i=0; for(;i<256&&fmap[i]==0;i++);;	if(i==256){i= -1;}	//壁判定
+				//絶対失敗判定
+				if(i<0 && tk.suf[0]!='*' && tk.suf[0]!='?'){ memset(clists[rpos], 0, 256); }
+				else {
+					//next有りのフツーのadd処理 !は最初に弾いてるので*?+が
+					// sucfailあるけどガバガバでも処理量が増えるだけで合法
+					for(int i=0;i<256;i++){
+						if(clists[rpos][i]!=1){ clists[rpos][i] = !!fmap[i]; }
 					}
 				}
-				//未消費系は追加で拾う			
-				if(tk.suf[0]=='*'||tk.suf[0]=='?'||(tk.ctg[0]=='l'&&tk.data[0]=='0')) {continue;}
-				break;	//消費系かsepとか
+			}
+lb_WALK:;			
+			//walk判定 *? なら追加有り +&系は完了 ! ""は個別に始末済
+			if(tk.suf[0]=='*'||tk.suf[0]=='?') {continue;}
+			
+			//未消費到達フラグ
+			if(tk.ctg[0]=='s') {
+				//0を-1で埋める
+				for(int i=0;i<256;i++){	clists[rpos][i]=(clists[rpos][i]||-1); }
+				fillrpos[rpos] = -1;	//carrが完了したruleflg -1は未消費1は消費
+			}
+			//消費フラグ eofは直接ここに流れ着く
+			if(fillrpos[rpos]==0){ fillrpos[rpos]=1; }
+
+			//jmpがあれば戻るし
+			if( idq_len(rstk) !=0){
+				rpos = idq_pop(rstk);
+				tpos = idq_pop(tstk)-1;	//top+1
+				lprpos[rpos]=0;
+				idx = rdata->rhead[rpos] -1;	//lua -1
+				continue;
+			}
+			//なければおしまい
+			break;
+		}
+	}
+	
+	//pass_2: 隠れ!ident*を検査
+	for(int rpos=0; rpos<erpos; rpos++) {
+		;STOP(fillrpos[rpos]==0, "fillrpos leak" );
+		int idx = rdata->rhead[rpos] -1;	//lua分 rpos0はheader
+		int cur=1; 	//1スタート 0はheadが入る
+		for(;;cur++) {
+			rtoken_t tk = rdata->rtokens[idx+cur];
+			if(tk.ctg[0]=='s') {break;}
+			if(tk.pre[0]&&tk.ctg[0]=='i'&&fillrpos[tk.num-1]<0){ cur= -cur; }
+			//存在しないはず 事前に弾いているはず
+			else if(tk.pre[0]&&tk.suf[0]){ cur= -cur; STOP(1, "miss" );}
+			if(cur<0){
+				char sbuf[256]={0};
+				sprintf(sbuf, "ERR: rule %s:%d,%d: \n"
+					"detect illigal '&!rule', no char consume grammar == !__*\n"
+					"&__* / !__?: always suc/fail"				
+					,rdata->rtokens[idx].data, idx+1, -cur);
+				emsg=strdup(sbuf);
+				free(clists);
+				clists=NULL;
+				goto lb_RTN;
 			}
 		}
-		if(lpflg==0) {break;}
-		cnt++;	//dbg用
 	}
-
-	// 最後に未消費マークがあれば自動成功 できるだけ弾きたいけど別ロジックが欲しい
-	for(int i=0;i<erpos;i++){ if(allmk[i]){ memset(clists[i], 1, 256); } }
-	free(allmk);
+//free
+lb_RTN:;
+	free(lprpos);
+	free(fillrpos);
+	idq_free(rstk);
+	idq_free(tstk);
 	rdata->clists=clists;
-	return NULL;	//noerr
+	return emsg;	//noerr==NULL
 }
+
 static void reset_clists(rdata_t* rdata) {
 	if(rdata==NULL){return;}
 	; if(!rdata->clists){return;}
 	; if(!rdata->rhead){return;}
-	for(int i=0; rdata->rhead[i]!= -9; i++) { free(rdata->clists[i]); }
+//	for(int i=0; rdata->rhead[i]!= -9; i++) { free(rdata->clists[i]); }
 	free(rdata->clists);
 	rdata->clists=NULL;
 }
 
+
+static char* clists_info(ped_t* ped){
+	sg_t* sg=sg_new();
+	clist_t* clists = ped->rdata->clists;
+	int rend = ped->rdata->norule_rpos;
+	for(int i=0;i<rend;i++){
+		int cnt=0;
+		int pflg=0;
+		for(int ii=0;ii<256;ii++){
+			if(clists[i][ii]==1){cnt++;}
+			if(clists[i][ii]==-1){pflg= -1;}
+			if(clists[i][ii]==-2){pflg= -2;}
+		}
+		const char* p =  gettoken(ped, i, 0)->data;
+		if(cnt==256){ sg_addf(sg, "rule %d %s: all 1\n", i+1, p); }
+		else if(cnt==0&&pflg==0){ sg_addf(sg, "rule %d %s: all 0\n", i+1, p); }
+		else if(cnt==0&&pflg){ sg_addf(sg, "rule %d %s : all -1\n", i+1, p); }
+		else if(cnt<=128){
+			sg_addf(sg, "rule %d %s: ", i+1, p);
+			for(int ii=0;ii<256;ii++){
+				if(clists[i][ii]>0){ sg_addf(sg, "%d:%d ", ii, clists[i][ii]);}
+			}
+			if(pflg){ sg_addf(sg, "others: %d\n", pflg); }
+			else{ sg_addf(sg, "others: 0\n"); }
+		}
+		else{
+			sg_addf(sg, "rule %d %s: ", i+1, p);
+			for(int ii=0;ii<256;ii++){
+				if(clists[i][ii]<=0){ sg_addf(sg, "%d:%d ", ii, clists[i][ii]);}
+			}
+			sg_addf(sg, "others: 1\n");
+		}
+	}
+	char* p = strdup(sg_ptr(sg));
+	sg_free(sg);
+	return p;
+}
+
+
 #define RESARR_INITSZ	128		//自動伸長タイプ
 static jst_t* jst_new(const char* nlstr, int nlstrsz){
 	char* emsg=NULL;
@@ -1162,7 +1346,7 @@ static void jst_reset(jst_t* jst){
 	jst->nlcnt=0;
 	jst->ccnt=0;
 	jst->rpos=0;
-	jst->tpos=1;
+	jst->tpos=0;
 	jst->seof=0;
 	jst->resepos = -1;
 	jst->ipos_c = -1;
@@ -1283,6 +1467,8 @@ static void pfix_restore(ped_t* ped) {
 
 static int fix_result(ped_t* ped, int rc) {
 	//大変。suf/preの分類.tokenの体裁も整える rtn 0/1/2 loop, suc,fail,eof 色々
+	//clistのスキップも加わった 操作前に確定失敗なのでtopでいい
+	if(rc== -3){return -2;}
 	rtoken_t* tk = gettoken(ped, -1, -1);
 	jst_t* jst = ped->jst;
 	if(rc== -1 && (tk->suf[0]=='*'||tk->suf[0]=='+') ) {
@@ -1417,10 +1603,8 @@ static int jmp_suc(ped_t* ped) {
 	//acc / vic判定	drule作成時にendに-1のtokenを付加してる
 	if( gettoken(ped, -1, -1)->num<0 ) {
 // 戻り先を覗き見. curが進んでないのに+*系だったら永久になるので弾く
-// 2団jmpでも最終的に*+系にたどり着くのでそこでkickできる
-// jmp系永久はここで弾ける reseposをpresetならpre_curposも始末出来るか
-		//jmp直後にipos_push(,...resepos)してるので.curposが取り出せる
-		//epos情報はaccもvicも共通で使うので先取り出来る？
+//jmp直後にipos_push(,...resepos)してるので.curposが取り出せる
+//epos情報はaccもvicも共通で使うので先取り出来る？
 		int pos = ipos_pop(ped->jst);
 		resarr_t res = jst->resarr[pos];
 		if(res.curpos == jst->curpos){
@@ -1448,7 +1632,7 @@ static int jmp_suc(ped_t* ped) {
 			//一致させておきたいところだけど
 			rc=0;
 			jst->rpos=0;
-			jst->tpos=1;
+			jst->tpos=0;
 			//量によって継続かeditか異なる
 			if(jst->curpos > ped->msv->bufsz_min ){ rc= -11; }	//報告なら-11, 継続は0
 			else if(jst->curpos<0) { rc=-10; jst->seof=2; }	//EOFヒット !. {_0 = "gw"} とか
@@ -1476,7 +1660,7 @@ static int jmp_fail(ped_t* ped) {
 		jst->rpos++;
 		// next有 小細工してjmp成功を演じる
 		if(grp==gettoken(ped, -1, -1)->num ) {
-			jst->tpos=1;
+			jst->tpos=0;
 			rc=0;
 			//jmp先をnextに更新	{mode,n,jst->rpos,jst->tpos,jst->curpos};
 			jst->resarr[jst->resepos].num++;	//直接関係ないがluaidx
@@ -1507,7 +1691,7 @@ static int jmp_fail(ped_t* ped) {
 		int left = sg_sz(jst->instr) - jst->curpos;
 		if(jst->seof==0 && left==0) {
 			jst->rpos--;
-			jst->tpos=1;
+			jst->tpos=0;
 			rc=ped->msv->bufsz_min;
 			goto lb_RTN;
 		}
@@ -1519,7 +1703,7 @@ static int jmp_fail(ped_t* ped) {
 		goto lb_RTN;
 	}
 	//endじゃないなら次に挑戦
-	jst->tpos=1;
+	jst->tpos=0;
 	rc=0;
 	goto lb_RTN;
 lb_RTN:;
@@ -1533,7 +1717,6 @@ static jbm_t ped_jmp(ped_t* ped) {
 	jbm_t res={0};
 	jst_t* jst = ped->jst;
 	
-	
 	//mainloop
 	int rc=0;
 	while(1) {
@@ -1555,24 +1738,60 @@ static jbm_t ped_jmp(ped_t* ped) {
 		if(jst->curpos < 0) { goto lb_FAIL; }	//EOF consumed. auto fail all
 		int leftsz = sg_sz(jst->instr) - jst->curpos;
 		if(jst->seof==0 && leftsz==0) { rc= BLKsz; continue; } //適当に追加
-		rtoken_t* tk = gettoken(ped, -1,-1);
 		//purne
-		if(jst->tpos== 1 && leftsz>0 ) {
+//dbg(jst->rpos, jst->tpos, tk->data);
+
+//loopだとtpos==1で何度も通ることになるからjmpとかloop以外のフラグが必須
+//vic fail identによるenterはtpos=0にセットして初回を確保する事にした
+		if(jst->tpos== 0 && leftsz>0 ) {
+			jst->tpos=1;
 			int c = (uint8_t)sg_ptr(jst->instr)[jst->curpos];
-			int rc = ped->rdata->clists[jst->rpos][c];	// cfilter
-// for(int i=0; i<256;i++){
-// 	printf("%d %d\n", i, ped->rdata->clists[jst->rpos][i]);
-// }
-// sleep(1);
-			if(rc==0) { goto lb_FAIL;}
+			int flg = ped->rdata->clists[jst->rpos][c];	// cfilter
+//dbg(c, rc, jst->rpos, jst->tpos, tk->data);
+/*
+for(int i=0; i<256;i++){
+	printf("%c:%d %d, %d %d\n",c,c,rc, i, ped->rdata->clists[jst->rpos][i]);
+}
+sleep(1);
+//*/
+			if(flg==1) {;}	//pass
+			else if(flg==0) {
+				rc= -3;	//clist専用 無条件rule失敗
+				continue;
+			}
+			else if(flg== -1) {
+				//中間は捏造。未消費成功はfailで飛ばすと自動的にsucになるはず
+				for(;;jst->tpos++ ){
+					if(gettoken(ped, -1, -1)->num<0){break;}
+					res_push(ped->jst, TK_LIT, 0);
+				};
+				jst->tpos--;	//sepから一つ前
+				jst->resepos--;	//FAIL先で""が追加される
+				goto lb_FAIL;
+			}
+			else if(flg== -2) {
+				//fatal 永久loopの発見だけど、ここで見つけなくても
+				//実行時検査で見つかるはず。一応作っといて両方テスト
+				//するのがいいか>>****否。ここで弾かないとダメ。
+				//本来のloopキックは全部loopck時に弾かなくてはならない
+				//お情けで危ないruleを見逃してるのでダメなcharがきたら
+				//弾かなくてはならない jmploopになってしまう
+				//未消費成功は見てるけどjmp時点ではnestの関係もあってみえない
+				//正常か異常かjmpの時点で判別できない
+				jmp_seterr(ped, "input byte causes infinite loop(dangerous rule)");
+				rc= -20;
+				break;
+			}
 		}
+		if(jst->tpos==0){jst->tpos=1;}
+		rtoken_t* tk = gettoken(ped, -1,-1);
 		// core start
 		if(tk->ctg[0]=='i') {
 			//ident
 			int rpos_dst = tk->num -1;	//luaは1startなのでずれる
 			res_push(ped->jst, TK_IDENT, rpos_dst);	//jmp先numを突っ込むが実際は使わない
 			jst->rpos=rpos_dst;
-			jst->tpos=1;
+			jst->tpos=0;
 			ipos_push(ped->jst, jst->resepos);	//jmpメモ
 			continue; //pre系はaccで戻ってきてからs.resarrで処理する rc=0でいい
 		}
@@ -1624,8 +1843,8 @@ static jbm_t ped_jmp(ped_t* ped) {
 			if(jst->seof==0) { rc = -30; continue; }		//full要求 regはlongest
 			ureg_t* rg = getrg(ped, -1, -1);
 			rc = ureg_search_head(rg, sg_ptr(jst->instr)+jst->curpos, leftsz);
+//dbg(rc, rg->p, rg->sz, jst->curpos, leftsz, jst->seof);sleep(1);
 			if(rc<0){ goto lb_FAIL; }
-//dbg(rg->p, rg->sz);sleep(1);
 			int sz = rg->sz;
 			res_push(ped->jst, TK_LIT, sz);
 			jst->curpos += sz;
@@ -2691,6 +2910,10 @@ int main(){
 /*
  change log
  --
+2021-07-22  Momi-g	<dmy@dmy.dmy>
+
+	* ped.c (ped_jmp): fix clist filter, add -2 etc
+
 2021-07-10  Momi-g	<dmy@dmy.dmy>
 
 	* ped.c (set_clists): fix grep ident fastmap, limit var idx >> nidx.
